@@ -1,10 +1,11 @@
 package Log::Any::Adapter::Log4perl;
 use Log::Log4perl;
+use Log::Any::Adapter::Util qw(make_method);
 use strict;
 use warnings;
 use base qw(Log::Any::Adapter::Base);
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 sub init {
     my ($self) = @_;
@@ -12,16 +13,49 @@ sub init {
     $self->{logger} = Log::Log4perl->get_logger( $self->{category} );
 }
 
-# Delegate methods to logger, mapping levels down to log4perl levels where necessary
-#
 foreach my $method ( Log::Any->logging_and_detection_methods() ) {
     my $log4perl_method = $method;
+
+    # Map log levels down to log4perl levels where necessary
+    #
     for ($log4perl_method) {
         s/notice/info/;
         s/warning/warn/;
         s/critical|alert|emergency/fatal/;
     }
-    __PACKAGE__->delegate_method_to_slot( 'logger', $method, $log4perl_method );
+
+    # Delegate to log4perl logger, increasing caller_depth so that %F, %C,
+    # etc. are generated correctly
+    #
+    make_method(
+        $method,
+        sub {
+            my $self = shift;
+            local $Log::Log4perl::caller_depth =
+              $Log::Log4perl::caller_depth + 1;
+            return $self->{logger}->$log4perl_method(@_);
+        }
+    );
+}
+
+# Override alias and printf variants to increase depth first
+#
+my %aliases = Log::Any->log_level_aliases;
+my @methods = (
+    keys(%aliases),
+    ( map { $_ . "f" } ( Log::Any->logging_methods, keys(%aliases) ) )
+);
+foreach my $method (@methods) {
+    make_method(
+        $method,
+        sub {
+            my $self = shift;
+            local $Log::Log4perl::caller_depth =
+              $Log::Log4perl::caller_depth + 2;
+            my $super_method = "SUPER::$method";
+            return $self->$super_method(@_);
+        }
+    );
 }
 
 1;
